function color_aligned = align_depth2color(camCalibData, rgbim, depthim)
bApplyDistort = true;
rgbcam.K = camCalibData.K_color;
rgbcam.distCoef = camCalibData.distCoeffs_color;

depthcam.Kdist = camCalibData.K_depth;
depthcam.distCoeffs = camCalibData.distCoeffs_depth;


[X,Y] = meshgrid( (0:511), (0:423) );
p2dd = [X(:) Y(:)];

% xp = unproject_2d([p2dd depth(:)], depthcam);
% xp = xp';

pn2d = ( depthcam.Kdist \ [p2dd'; ones(1, size(p2dd,1))] )';
k = [reshape(depthcam.distCoeffs(1:5),[],1); zeros(12-5,1)];
x0 = pn2d(:,1); 
y0 = pn2d(:,2);
x = x0; y = y0;

% Undistortion iterations.
% See cv::undistortPoints n modules/imgproc/src/undistort.cpp
% https://github.com/opencv/opencv
for iter=1:5
    r2 = x.*x + y.*y;
    icdist = (1 + ((k(1+7)*r2 + k(1+6)).*r2 + k(1+5)).*r2)./(1 + ((k(1+4)*r2 + k(1+1)).*r2 + k(1+0)).*r2);
    deltaX = 2*k(1+2)*x.*y + k(1+3)*(r2 + 2*x.*x)+ k(1+8)*r2+k(1+9)*r2.*r2;
    deltaY = k(1+2)*(r2 + 2*y.*y) + 2*k(1+3)*x.*y+ k(1+10)*r2+k(1+11)*r2.*r2;
    x = (x0 - deltaX).*icdist;
    y = (y0 - deltaY).*icdist;
end
xp = [x y]';

%Apply distortion
%Distortion parameter order follows opencv format
%(http://docs.opencv.org/2.4/doc/tutorials/calib3d/camera_calibration/camera_calibration.html)
if bApplyDistort
    X2 = xp(1,:).*xp(1,:);
    Y2 = xp(2,:).*xp(2,:);
    XY = xp(1,:).*xp(2,:);
    r2 = X2+Y2;
    r4 = r2.*r2;
    r6 = r2.*r4;

    Kp = rgbcam.distCoef;
    radial       = 1.0 + Kp(1)*r2 + Kp(2)*r4 + Kp(5)*r6;
    tangential_x = 2.0*Kp(3)*XY + Kp(4)*(r2 + 2.0*X2);
    tangential_y = 2.0*Kp(4)*XY + Kp(3)*(r2 + 2.0*Y2);

    xp = [radial;radial].*xp(1:2,:) + [tangential_x; tangential_y];
end

p2d = bsxfun(@plus, rgbcam.K(1:2,1:2)*xp, rgbcam.K(1:2,3))';

%% Project 3D points (from depth) to color image
color_aligned = multiChannelInterp( double(rgbim)/255, ...
    p2d(:,1)+1, p2d(:,2)+1, 'linear');

color_aligned = reshape(color_aligned, [size(depthim,1), size(depthim,2), 3]);

