root_path = '/media/srivasta/predictor2_data1/work/Projects/panoptic-toolbox/kinect-data'; %Put your root path where sequence folders are locates
seqName = '170307_dance5';  %160906_band1,Put your target sequence name here 

hd_index_list= 500:500; % Target frames you want to export ply files

%The followings are an example
% root_path = '/posefs0c/panoptic' %An example
% seqName = '160422_haggling1' %An example
% hd_index_list= 500:510; % Target frames you want to export ply files

%Relative Paths
kinectImgDir = sprintf('%s/%s/kinectImgs',root_path,seqName);  
kinectDepthDir = sprintf('%s/%s/kinect_shared_depth',root_path,seqName);
calibFileName = sprintf('%s/%s/kcalibration_%s.json',root_path,seqName,seqName);
syncTableFileName = sprintf('%s/%s/ksynctables_%s.json',root_path,seqName,seqName);
panopcalibFileName = sprintf('%s/%s/calibration_%s.json',root_path,seqName,seqName);
panopSyncTableFileName = sprintf('%s/%s/synctables_%s.json',root_path,seqName,seqName);
panopSyncTableFileName = sprintf('%s/%s/synctables_%s.json',root_path,seqName,seqName);
poses3d_dir = sprintf('%s/%s/hdPose3d_stage1_coco19',root_path,seqName);



%Other parameters
bVisOutput = 1; %Turn on, if you want to visualize what's going on
addpath('jsonlab');
addpath('kinoptic-tools');

%% Load syncTables
ksync = loadjson(syncTableFileName);
knames = {};
for id=1:10; knames{id} = sprintf('KINECTNODE%d', id); end
psync = loadjson(panopSyncTableFileName); %%Panoptic Sync Tables

%% load the 3d poses
% dinfo = dir(sprintf('%s/*.json', poses3d_dir));
% poses3d = [];
% for K = 1 : length(dinfo)
%   thisfilename = dinfo(K).name;  %just the name
%   json_file = [dinfo(K).folder '/' dinfo(K).name];
%   json.num = str2num(strrep(strrep(dinfo(K).name, 'body3DScene_',''), '.json', ''));
%   json.data = loadjson(json_file);
%   poses3d = [poses3d; json];
% end
%% Load Kinect Calibration File
kinect_calibration = loadjson(calibFileName); 
panoptic_calibration = loadjson(panopcalibFileName);
panoptic_camNames = cellfun( @(X) X.name, panoptic_calibration.cameras, 'uni', false ); %To search the targetCam


hd_index_list = hd_index_list+2; %This is the output frame (-2 is some weired offset in synctables)


for hd_index = hd_index_list
    %% Compute Universal time
    selUnivTime = psync.hd.univ_time(hd_index);
    fprintf('hd_index: %d, UnivTime: %.3f\n', hd_index, selUnivTime)
    %% Main Iteration    
    all_point3d_panopticWorld = []; %point cloud output from all kinects
    all_colorsv = [];   %colors for point cloud 
    for idk = 1:10  %Iterating 10 kinects. Change this if you want a subpart
        if idk==1 && bVisOutput   %Visualize the results from the frist kinect only. 
            vis_output = 1;
        else
            vis_output = 0;
        end
        %% Select corresponding frame index rgb and depth by selUnivTime
        % Note that kinects are not perfectly synchronized (it's not possible),
        % and we need to consider offset from the selcUnivTime
        [time_distc, cindex] = min( abs( selUnivTime - (ksync.kinect.color.(knames{idk}).univ_time-6.25) ) );  %cindex: 1 based
        ksync.kinect.color.(knames{idk}).univ_time(cindex);
        % assert(time_dist<30);
        [time_distd, dindex] = min( abs( selUnivTime - ksync.kinect.depth.(knames{idk}).univ_time ) ); %dindex: 1 based

        % Filtering if current kinect data is far from the selected time
        fprintf('idk: %d, %.4f\n', idk, selUnivTime - ksync.kinect.depth.(knames{idk}).univ_time(dindex));
        if abs(ksync.kinect.depth.(knames{idk}).univ_time(dindex) - ksync.kinect.color.(knames{idk}).univ_time(cindex))>6.5
            fprintf('Skipping %d, depth-color diff %.3f\n',  abs(ksync.kinect.depth.(knames{idk}).univ_time(dindex) - ksync.kinect.color.(knames{idk}).univ_time(cindex)));    
            continue;
        end
        % assert(time_dist<30);
        % time_distd
        if time_distc>30 || time_distd>17 
            fprintf('Skipping %d\n', idk);
            [time_distc, time_distd];
            continue;
        end

        % Extract image and depth
        %rgbim_raw = kdata.vobj{idk}.readIndex(cindex); % cindex: 1 based
        rgbFileName = sprintf('%s/50_%02d/50_%02d_%08d.jpg',kinectImgDir,idk,idk,cindex);
        depthFileName = sprintf('%s/KINECTNODE%d/depthdata.dat',kinectDepthDir,idk);

        rgbim = imread(rgbFileName); % cindex: 1 based
        %depthim_raw = kdata.vobj{idk}.readDepthIndex(dindex);  % cindex: 1 based
        depthim = readDepthIndex_1basedIdx(depthFileName,dindex);  % cindex: 1 based
        
        depthim_new =  uint8((depthim - min(min(depthim)))./ (max(max(depthim)) - min(min(depthim)))*255);
        Map = jet(255);
        depthim_new = ind2rgb(depthim_new, Map);

        %% Back project depth to 3D points (in camera coordinate)
        camCalibData = kinect_calibration.sensors{idk};
        % point3d (N x 3): 3D point cloud from depth map in the depth camera cooridnate
        % point2d_color (N x 2): 2D points projected on the rgb image space
        % Where N is the number of pixels of depth image (512*424)
        aligned_color = align_depth2color(camCalibData, rgbim, depthim);
        
                
        fused = imfuse(aligned_color, depthim_new ,'blend','Scaling','joint');
        if vis_output
            % figure; imshow(rgbim);     title('Original RGB Image');
            figure; imshow(depthim_new);  title('Original Depth Image');
            figure; imshow(aligned_color); title('Aligned Color Image');
            figure; imshow(fused); title('fused ');
        end

    end
end

